
extern vec2 size = vec2(480,264);
extern number samples = 2;
extern number quality = .1;
extern number gametime = 0;
extern number intensity = 1;

vec3 rgb2hsv(vec3 c)
{
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));

    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

vec3 hsv2rgb(vec3 c)
{
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

vec4 bloom(vec4 colour, Image tex, vec2 tc, vec2 sc)
{
  vec4 img = Texel(tex, tc);
  vec4 sum = vec4(0);
  int diff = (samples - 1) / 2;
  vec2 sizeFactor = vec2(1) / size * quality;

  for (int x = -diff; x <= diff; x++)
  {
    for (int y = -diff; y <= diff; y++)
    {
      vec2 offset = vec2(x, y) * sizeFactor;
      sum += Texel(tex, tc + offset);
    }
  }

  return ((sum / (float)(samples * samples)) + img) * colour;
}

vec4 effect( vec4 color, Image texture, vec2 texture_coords, vec2 screen_coords ){
    texture_coords.x += cos(gametime*intensity+texture_coords.y*10.0)/500.0;

    vec4 pixel = bloom(color, texture, texture_coords, screen_coords);

	float amount = distance(texture_coords, vec2(0.5,0.5));


    color.a = pixel.a;
	pixel = mix(pixel * color, vec4(0,0,0,pixel.a), amount*1.2);
	vec3 col = rgb2hsv(vec3(pixel.r, pixel.g, pixel.b));
	col.g *= 1.2;
	col = hsv2rgb(col);

	return vec4(col.r, col.g, col.b, pixel.a);

}
