
require "entities/player_trigger"
require "entities/entity"
require "model"

Chest = Entity:extend("Chest")

function Chest:init(x, y)

    self.model = Model:new("chest", 32)
    self.x = x
    self.y = y

    self.physics = Physics:new(self.x, self.y, "static", 32, 32)

    self.trigger = PlayerTrigger:new(self.x-16, self.y-16, 64, 64)
    self.trigger.trigger = function()
        game.lock = self
    end

    self.numb1 = 0
    self.numb2 = 0
    self.numb3 = 0
    self.changeTimer = 0
    self.canClick = true

    self.step = 1
end

function Chest:update(dt)

    self.trigger:update(dt)

    if game.lock == self then
        if not self.trigger.hasPlayer then
            game.lock = nil
        elseif love.mouse.isDown(1) or love.mouse.isDown(2) then
            local inc = love.mouse.isDown(1) and 1 or -1
            if self.canClick then
                local mx, my = love.mouse.getPosition()
                local sx = GAME_WIDTH/2-10
                local sy = GAME_HEIGHT/2

                if CheckCollision(mx, my, 1, 1, sx-40, sy, 20, 80) then
                    self.numb1 = self.numb1+inc
                    self.canClick = false
                    audio:play("lock")
                end

                if CheckCollision(mx, my, 1, 1, sx, sy, 20, 80) then
                    self.numb2 = self.numb2+inc
                    self.canClick = false
                    audio:play("lock")
                end

                if CheckCollision(mx, my, 1, 1, sx+40, sy, 20, 80) then
                    self.numb3 = self.numb3+inc
                    self.canClick = false
                    audio:play("lock")
                end
            end

        else
            self.canClick = true
        end

        if self.numb1>9 then self.numb1 = 0 end
        if self.numb2>9 then self.numb2 = 0 end
        if self.numb3>9 then self.numb3 = 0 end
        if self.numb1<0 then self.numb1 = 9 end
        if self.numb2<0 then self.numb2 = 9 end
        if self.numb3<0 then self.numb3 = 9 end

        if (self.numb1==2 and self.numb2==3 and self.numb3==6) or (self.numb1==6 and self.numb2==2 and self.numb3==3) then
            game.lock = nil
            self.trigger.trigger = function() end
            game.dialog:show("\nThe chest opened!;\nThere are a couple cans of food inside.\n\nThe storm is here! I must get to shelter!")
            game.weather.intensity = 4
            game.fade = .2
            game.wind = love.audio.newSource("assets/audio/wind.ogg", "static")
            game.wind:setLooping(true)
            game.wind:play()
            audio:play("open")
        end

    else
        self.step = 1
    end

end

function Chest:drawLock()

    love.graphics.draw(sprites:get("lock"), GAME_WIDTH/2, GAME_HEIGHT/2, 0, 2, 2, 64, 64)

    local sx = GAME_WIDTH/2-10
    local sy = GAME_HEIGHT/2+30

    love.graphics.setFont(font2)
    love.graphics.print(self.numb1, sx-40, sy)
    love.graphics.print(self.numb2, sx+2, sy)
    love.graphics.print(self.numb3, sx+43, sy)
    love.graphics.setFont(font)

end
