require "entities/crate"
require "entities/physics"
require "entities/roof"
require "model"

EntityHandler = class("EntityHandler")

EntityHandler.ents = {}
EntityHandler.id = 0

function EntityHandler:init()

    self:add(Roof:new(1264, 737))

end

function EntityHandler:crates()
    for i=1,100 do
        table.insert(self.ents, Crate:new(love.math.random(0,GAME_WIDTH), love.math.random(0,GAME_WIDTH)))
    end
end

function EntityHandler:add(ent)
    table.insert(self.ents, ent)
    ent.id = self.id
    self.id = self.id+1
end

function EntityHandler:update(dt)

    for i,v in ipairs(self.ents) do
        v:calculateDepth()
        v:update(dt)

        if v.remove then
            v:destroy()
            table.remove(self.ents, i)
        end
    end

end

function EntityHandler:calculateDepth()
    for i,v in ipairs(self.ents) do
        v:calculateDepth()
    end
end

function sort(a, b)
    if a.front and not b.front then
        return false
    elseif b.front and not a.front then
        return true
    elseif a.depth < b.depth then
        return true
    elseif a.depth>b.depth then
        return false
    else
        return a.id>b.id
    end
end

function EntityHandler:draw()

    local inBound = {}
    for i,v in ipairs(self.ents) do
        if not v.remove and util.distSquared(v.x, v.y, game.camera.x-GAME_WIDTH/2, game.camera.y-GAME_HEIGHT/2) < GAME_WIDTH*GAME_WIDTH*1.05 then
            table.insert(inBound, v)
        end
    end

    table.sort(inBound, sort)
    for i=1,#inBound do
        inBound[i]:draw()
    end


end
