
Player = Entity:extend("Player")

function Player:init()
    self.x = 1770
    self.y = 980
    self.speed = 100
    self.sprite = "player"
    self.physics = Physics:new(self.x, self.y, "dynamic", "circle", 15)
    self.physics.body:setMass(1)
    self.physics:setEntity(self)
    self.physics.fixture:setCategory(2)

    self.idle = {delay=.3}
    self.walk = {delay=.2}
    self.carryAnim = {delay=.24}
    for i=1,4 do
        self.idle[i] = love.graphics.newQuad(i*32 - 32, 0, 32, 32, sprites:get("player"):getWidth(), sprites:get("player"):getHeight())
    end
    for i=1,4 do
        self.walk[i] = love.graphics.newQuad(i*32 - 32, 32, 32, 32, sprites:get("player"):getWidth(), sprites:get("player"):getHeight())
    end
    for i=1,4 do
        self.carryAnim[i] = love.graphics.newQuad(i*32 - 32, 64, 32, 32, sprites:get("player"):getWidth(), sprites:get("player"):getHeight())
    end
    self.animationTimer = 0
    self.animation = self.idle
    self.index = 1

    self.carry = nil
end

function Player:update(dt)
    local velx = 0
    local vely = 0
    local faceDir = 0

    if love.keyboard.isDown("w") then
        vely = vely - game.camera.cos*self.speed
        velx = velx - game.camera.sin*self.speed
    end
    if love.keyboard.isDown("a") then
        vely = vely + game.camera.sin*self.speed
        velx = velx - game.camera.cos*self.speed
        faceDir = faceDir-1
    end
    if love.keyboard.isDown("s") then
        vely = vely + game.camera.cos*self.speed
        velx = velx + game.camera.sin*self.speed
    end
    if love.keyboard.isDown("d") then
        vely = vely - game.camera.sin*self.speed
        velx = velx + game.camera.cos*self.speed
        faceDir = faceDir+1
    end

    if velx~=0 or vely~=0 then
        self.animation = self.walk

            local dir = math.atan2(vely, velx)
            velx = math.cos(dir)*self.speed
            vely = math.sin(dir)*self.speed

    else
        self.animation = self.idle
    end

    if faceDir~=0 then
        self.faceRight = faceDir==1
    end


    if self.carry then
        velx = velx/1.25
        vely = vely/1.25
        self.carry.rotation = self.carry.rotation+dt

        self.animation = self.carryAnim

        if velx==0 and vely==0 then
            self.index = 2
            self.animationTimer = 0
        end

    end

    self.physics:setSpeed(velx, vely)
    self.physics:updatePosition(self)

    if game.progress.animate then
        self.animation = self.walk
    end

    self:animate(dt)
end

function Player:animate(dt)
    self.animationTimer = self.animationTimer+dt
    if self.animationTimer>self.animation.delay then
        self.animationTimer = 0
        self.index = self.index+1
        if self.index>#self.animation then
            self.index = 1
        end

        if self.index%2==0 and self.animation ~= self.idle then
            if CheckCollision(self.x-16, self.y-16, 32, 32, 1143, 663, 1417-1143, 809-663+20) then
            --x: 1143 - 1417
            --y: 663 - 809
                audio:play("step_wood", love.math.random(80,120)/100)
            else
                audio:play("step", love.math.random(80,120)/100)
            end
        end
    end
end

function Player:draw()
    local spr = sprites:get(self.sprite)
    love.graphics.draw(spr, self.animation[self.index], math.floor(self.x), math.floor(self.y), -game.camera.rotation, self.faceRight and 1 or -1, 1, 16, 32)
end
