require "entities/entity_handler"
require "entities/player"
require "camera"
require "world"
require "dialog"
require "weather"
require "game_over"

Game = class("Game")

local intro = {
    "The Cold Cabin",
    "a Ludum Dare 37 game",
    "Created by Evgiz"
}

function Game:init()
    self.gameOver = GameOver:new(true)
    self.fade = 1
end

function Game:start()

    self.entities = EntityHandler:new()
    self.world = World:new()
    self.player = Player:new()
    self.camera = Camera:new()
    self.weather = Weather:new()

    self.world:load()

    self.entities:add(self.player)

    self.dialog = Dialog:new()

    flux.to(self.player, 10, {x = 1330, y=980}):ease("linear"):after(game.player, 1, {y=900}):ease("linear"):oncomplete(
    function()
        self.progress.intro = true
        self.player.animation = self.player.idle
        self.player.physics.body:setPosition(game.player.x, game.player.y)
        self.dialog:show("Finally!; I've found shelter.;;\n\nI need to find something\nto keep warm though.;;\n\nOtherwise I won't survive long in this\nweather...; ")
        flux.to(self.camera, 4, {rotation=math.rad(15)}):delay(.5)
    end)

    flux.to(self, 3, {fade=0})
    self.fade = .9

    flux.to(self, .5, {introAlpha=1}):delay(.5):after(self, .5, {introAlpha=0}):delay(2):oncomplete(
    function()
        self.intro = intro[2]
        flux.to(self, .5, {introAlpha=1}):delay(.5):after(self, .5, {introAlpha=0}):delay(2):oncomplete(
        function()
            self.intro = intro[3]
            flux.to(self, .5, {introAlpha=1}):delay(.5):after(self, .5, {introAlpha=0}):delay(2):oncomplete(
                function() self.intro = nil end
            )
        end
        )
    end
    )

    self.intro = intro[1]
    self.introAlpha = 0

    self.player.animation = self.player.walk

    self.progress = {}
    self.fade = 1


end

function Game:update(dt)
    if self.gameOver ~= nil then
        self.gameOver:update(dt)
        return
    end

    self.dialog:update(dt)
    self.camera:update(dt)
    self.weather:update(dt)

    if not self.progress.intro or self.dialog.open then
        self.entities:calculateDepth()

        if not self.progress.intro then
            self.player:animate(dt)
            self.camera.rotation = 0
        end
        return
    end

     if self.lock then
         self.lock:update(dt)
     end

    self.entities:update(dt)
    self.world:update(dt)

    --End game
    if self.weather.intensity==4 and util.distSquared(self.player.x, self.player.y, 1280, 750) < 50*50 then
        self.weather.intensity = 5
        self.dialog:show("\n\nThe storm is so strong!\n\nI have to wait out here\nand hope for the best!;  ")
    end

    if self.weather.intensity==5 and not self.dialog.open and not self.progress.fin then
        --Walls blow away
        self.progress.fin = true
        for i=1,#self.entities.ents do
            local ent = self.entities.ents[i]
            if ent.name=="Wall" or ent.name=="Bed" then
                flux.to(ent, 3.5, {y=ent.y+love.math.random(-100, 100), x=ent.x+GAME_WIDTH*2, rotation=ent.rotation+5}):delay(love.math.random(400,500)/100.0)
            elseif ent.name=="Roof" then
                ent.remove = true
            end
        end
        flux.to(self, 1, {fade=1}):delay(4.5)
        flux.to(self, 4, {}):oncomplete(function() audio:play("destroy") music:stop() end)
        flux.to(self, 0, {}):delay(6):oncomplete(function() self.gameOver = GameOver:new() end)
        self.intro = "The Cold Cabin\n"

        self.fade = .2
    end

    if self.weather.intensity==5 then
        self.camera.x = self.camera.x + love.math.random(-3, 3)
        self.camera.y = self.camera.y + love.math.random(-3, 3)
    elseif self.weather.intensity==4 then
        self.camera.x = self.camera.x + love.math.random(-1, 1)
        self.camera.y = self.camera.y + love.math.random(-1, 1)
    end
    --

end

function Game:draw()
    if self.gameOver~=nil then return end

    local rotAdd = 0
    if self.weather.intensity==5 then
        rotAdd = math.rad(love.math.random(-20,20)/10)
        self.camera.rotation = self.camera.rotation + rotAdd
    end

    self.camera:translate()
    self.world:draw()
    self.entities:draw()

    love.graphics.origin()

    self.camera.rotation = self.camera.rotation-rotAdd

    self.weather:draw()

    if self.fade>0 then
        love.graphics.setColor(0, 0, 0, self.fade*255)
        love.graphics.rectangle("fill", 0, 0, GAME_WIDTH, GAME_HEIGHT)
        love.graphics.setColor(255, 255, 255, 255)
    end

end

function Game:drawUI()

    if self.gameOver ~= nil then
        love.graphics.setColor(0, 0, 0, 255)
        love.graphics.rectangle("fill", 0, 0, GAME_WIDTH, GAME_HEIGHT)
        love.graphics.setColor(255, 255, 255, 255)
        self.gameOver:draw()
        return
    end

    self.dialog:draw()
    self.world:drawUI()

    if self.lock then
        self.lock:drawLock()
    end

    if self.intro ~= nil then
        love.graphics.setFont(font2)

        if not self.progress.fin then
            love.graphics.setColor(0, 0, 0, 255*self.introAlpha)
        else
            love.graphics.setColor(255, 255, 255, 255*self.introAlpha)
        end

        love.graphics.print(self.intro, GAME_WIDTH/2 - font2:getWidth(self.intro)/2, GAME_HEIGHT/4)
        love.graphics.setColor(0, 0, 0, 255)
        if not self.progress.fin then
            local str = "WASD + ARROWS"
            love.graphics.print(str, GAME_WIDTH/2 - font2:getWidth(str)/2, GAME_HEIGHT-30)
        end
        love.graphics.setColor(255, 255, 255, 255)
        love.graphics.setFont(font)
    end

end
