require "libraries/tablesaveload"

--Entities--------
require "entities/grass"
require "entities/crate"
require "entities/birch"
require "entities/wall"
require "entities/logs"
require "entities/fireplace"
require "entities/minor/bed"
require "entities/fence"
require "entities/fence_door"
require "entities/key"
require "entities/letter"
require "entities/chest"
------------------

World = class("World")

require "libraries/serialize"
local sti = require "libraries/sti"

local entityTypes = {
    Grass,
    Crate,
    Birch,
    Wall,
    Logs,
    FirePlace,
    Bed,
    Fence,
    FenceDoor,
    Key,
    Letter,
    Chest,
}

function World:init()
    self.map = sti("assets/level/level.lua")
    self.entities = {}
    self.edit = false
    self.placed = false
    self.saveTimer = 0
    self.placing = 1
    self.placeRotation = 0

    --Physics
    self.world = love.physics.newWorld(0, 0, true)
    love.physics.setMeter(32)

end

function World:load()
    --self.entities = table.load("level.save") or {}
    self.entities = serialize.load("assets/level/entities") or table.load("level.save") or {}
    for i=1,#self.entities do
        local ent
        for k,v in pairs(entityTypes) do
            if v.name == self.entities[i].name then
                ent = v:new(self.entities[i].x, self.entities[i].y, self.entities[i].rotation or 0)
                self.entities[i].entity = ent
                ent:setRotation(self.entities[i].rotation)
                game.entities:add(ent)
                break;
            end
        end
    end
end

function World:update(dt)
    self.world:update(dt)

    if self.edit then
        game.camera.rotation = 0

        if love.keyboard.isDown("left") then
            self.placeRotation = self.placeRotation-dt/2
        end
        if love.keyboard.isDown("right") then
            self.placeRotation = self.placeRotation+dt/2
        end

        self.saveTimer = self.saveTimer+dt

        if self.saveTimer>3 then
            self.saveTimer = 0
            table.save(self.entities, "level.save")
            serialize.save(self.entities,"entities")
        end
    end

    --if love.keyboard.isDown("e") then self.edit = true end
    --if love.keyboard.isDown("q") then self.edit = false end

    if self.edit and love.mouse.isDown(1) then
        if not self.placed then
            self.placed = true
            self:placeEntity()
        end
    else
        self.placed = false
    end

    if self.edit and love.mouse.isDown(2) then
        self:removeEntity()
    end

end

function World:wheel(y)
    self.placing = self.placing+1
    if self.placing>#entityTypes then
        self.placing = 1
    end
end

function World:removeEntity()
    local x = love.mouse.getX()/game.camera.zoom + game.camera.x - GAME_WIDTH/2/game.camera.zoom
    local y = love.mouse.getY()/game.camera.zoom + game.camera.y - GAME_HEIGHT/2/game.camera.zoom

    local min = 10*10
    local index = nil

    for i=1, #self.entities do
        local ent = self.entities[i]
        local dist = util.distSquared(ent.x or 0, ent.y or 0, x, y)
        if dist < min then
            min = dist
            index = i
        end
    end

    if index ~= nil then
        if self.entities[index].entity~=nil then
            self.entities[index].entity.remove = true
        end
        table.remove(self.entities, index)
    end

end

function World:placeEntity()
    game.camera.rotation = 0

    local xx = love.mouse.getX()/game.camera.zoom + game.camera.x - GAME_WIDTH/2/game.camera.zoom
    local yy = love.mouse.getY()/game.camera.zoom + game.camera.y - GAME_HEIGHT/2/game.camera.zoom
    local rot = math.deg(self.placeRotation)

    if love.keyboard.isDown("lctrl") then
        xx = math.floor(xx/32)*32
        yy = math.floor(yy/32)*32
        rot = math.floor(rot/45)*45
    end

    local ent = entityTypes[self.placing]:new(xx,yy)
    local data = {name=ent.name, x=xx, y=yy, rotation=math.rad(rot), entity=ent}

    ent.rotation = math.rad(rot)

    table.insert(self.entities, data)
    game.entities:add(ent)

end

function World:draw()
    self.map:draw()

    if self.edit then
        love.graphics.setColor(255, 0, 0, 100)
        for i,v in ipairs(self.entities) do
            love.graphics.circle("fill", v.x or 0, v.y or 0, 10, 10)
        end
        love.graphics.setColor(255, 255, 255, 255)
    end
end

function World:drawUI()

    if self.edit then
        love.graphics.print("Place: "..entityTypes[self.placing].name, 0, 12)
        love.graphics.print("Rot: "..math.deg(self.placeRotation), 0, 24)
    end

end
